package com.greenorange.promotion.controller.project;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.greenorange.promotion.annotation.RequiresPermission;
import com.greenorange.promotion.annotation.SysLog;
import com.greenorange.promotion.common.BaseResponse;
import com.greenorange.promotion.common.ErrorCode;
import com.greenorange.promotion.common.ResultUtils;
import com.greenorange.promotion.constant.UserConstant;
import com.greenorange.promotion.exception.ThrowUtils;
import com.greenorange.promotion.model.dto.CommonBatchRequest;
import com.greenorange.promotion.model.dto.project.ProjectAddRequest;
import com.greenorange.promotion.model.dto.project.ProjectQueryRequest;
import com.greenorange.promotion.model.dto.project.ProjectUpdateRequest;
import com.greenorange.promotion.model.entity.Project;
import com.greenorange.promotion.model.vo.project.ProjectVO;
import com.greenorange.promotion.service.common.CommonService;
import com.greenorange.promotion.service.project.ProjectService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.greenorange.promotion.model.dto.CommonRequest;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * 项目 控制器
 */
@RestController
@RequestMapping("project")
@Slf4j
@Tag(name = "项目管理")
public class ProjectController {

    @Resource
    private ProjectService projectService;

    @Resource
    private CommonService commonService;

    /**
     * web端管理员添加项目
     * @param projectAddRequest 项目添加请求体
     * @return 是否添加成功
     */
    @PostMapping("add")
    @Operation(summary = "web端管理员添加项目", description = "参数：项目添加请求体，权限：管理员，方法名：addProject")
    public BaseResponse<Boolean> addProject(@Valid @RequestBody ProjectAddRequest projectAddRequest) {
        Project project = commonService.copyProperties(projectAddRequest, Project.class);
        projectService.save(project);
        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id修改项目信息
     * @param projectUpdateRequest 项目更新请求体
     * @return 是否更新成功
     */
    @PostMapping("update")
    @Operation(summary = "web端管理员更新项目", description = "参数：项目更新请求体，权限：管理员，方法名：updateProject")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员根据id修改项目信息")
    public BaseResponse<Boolean> updateProject(@Valid @RequestBody ProjectUpdateRequest projectUpdateRequest) {
        Project project = commonService.copyProperties(projectUpdateRequest, Project.class);
        projectService.updateById(project);
        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id删除项目
     * @param commonRequest 项目删除请求体
     * @return 是否删除成功
     */
    @PostMapping("delete")
    @Operation(summary = "web端管理员根据id删除项目", description = "参数：项目删除请求体，权限：管理员，方法名：delProject")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员根据id删除项目")
    public BaseResponse<Boolean> delProject(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        projectService.removeById(id);
        return ResultUtils.success(true);
    }

    /**
     * web端管理员批量删除项目
     * @param commonBatchRequest 项目批量删除请求体
     * @return 是否删除成功
     */
    @PostMapping("delBatch")
    @Operation(summary = "web端管理员批量删除项目", description = "参数：项目批量删除请求体，权限：管理员，方法名：delBatchProject")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员批量删除项目")
    public BaseResponse<Boolean> delBatchProject(@Valid @RequestBody CommonBatchRequest commonBatchRequest) {
        List<Long> ids = commonBatchRequest.getIds();
        projectService.removeByIds(ids);
        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id查询项目
     * @param commonRequest 项目查询请求体
     * @return 项目信息
     */
    @PostMapping("queryById")
    @Operation(summary = "web端管理员根据id查询项目", description = "参数：项目查询请求体，权限：管理员，方法名：queryProjectById")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员根据id查询项目")
    public BaseResponse<ProjectVO> queryProjectById(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        Project project = projectService.getById(id);
        ThrowUtils.throwIf(project == null, ErrorCode.OPERATION_ERROR, "当前项目不存在");
        ProjectVO projectVO = commonService.copyProperties(project, ProjectVO.class);
        return ResultUtils.success(projectVO);
    }

    /**
     * Web端管理员分页查询项目
     * @param projectQueryRequest 项目查询请求体
     * @return 项目列表
     */
    @PostMapping("page")
    @Operation(summary = "Web端管理员分页查询项目", description = "参数：项目查询请求体，权限：管理员，方法名：listProjectByPage")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "Web端管理员分页查询项目")
    public BaseResponse<Page<ProjectVO>> listProjectByPage(@Valid @RequestBody ProjectQueryRequest projectQueryRequest) {
        long current = projectQueryRequest.getCurrent();
        long pageSize = projectQueryRequest.getPageSize();
        QueryWrapper<Project> queryWrapper = projectService.getQueryWrapper(projectQueryRequest);
        Page<Project> page = projectService.page(new Page<>(current, pageSize), queryWrapper);
        List<Project> projectList = page.getRecords();
        List<ProjectVO> projectVOList = commonService.convertList(projectList, ProjectVO.class);
        Page<ProjectVO> voPage = new Page<>(current, pageSize);
        voPage.setRecords(projectVOList);
        voPage.setPages(page.getPages());
        voPage.setTotal(page.getTotal());
        return ResultUtils.success(voPage);
    }
}