package com.greenorange.promotion.model.dto.project;

import com.greenorange.promotion.annotation.EnumValue;
import com.greenorange.promotion.model.enums.ProjectStatusEnum;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.DecimalMin;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;

/**
 * 项目添加请求体
 */
@Data
@Schema(description = "项目添加请求体", requiredProperties = {
                "projectName",
                "projectImage",
                "maxProjectPrice",
                "minProjectPrice",
                "projectDescription",
                "settlementDesc",
                "projectDesc",
                "projectFlow",
                "applyPromoCodeDesc",
                "projectSettlementCycle",
                "currentPromotionCount",
                "maxPromoterCount",
                "projectStatus",
                "isShelves",
})
public class ProjectAddRequest implements Serializable {

    /**
     * 项目名称
     */
    @NotBlank(message = "项目名称不能为空")
    @Schema(description = "项目名称", example = "美团省钱包")
    private String projectName;

    /**
     * 项目图片URL
     */
    @NotBlank(message = "项目图片URL不能为空")
    @Schema(description = "项目图片URL", example = "http://xxx.png")
    private String projectImage;

    /**
     * 项目最高价格
     */
    @DecimalMin(value = "0", message = "项目最低价格不能小于0")
    @Schema(description = "项目最高价格", example = "30")
    private BigDecimal maxProjectPrice;

    /**
     * 项目最低价格
     */
    @DecimalMin(value = "0", message = "项目最低价格不能小于0")
    @Schema(description = "项目最低价格", example = "10")
    private BigDecimal minProjectPrice;

    /**
     * 项目简介
     */
    @NotBlank(message = "项目简介不能为空")
    @Schema(description = "项目简介", example = "不限制推广方式，禁止/恶意/夸大虚假宣传")
    private String projectDescription;

    /**
     * 结算说明（富文本）
     */
    @NotBlank(message = "结算说明（富文本）不能为空")
    @Schema(description = "结算说明（富文本）", example = "富文本")
    private String settlementDesc;

    /**
     * 项目说明（富文本）
     */
    @NotBlank(message = "项目说明（富文本）不能为空")
    @Schema(description = "项目说明（富文本）", example = "富文本")
    private String projectDesc;

    /**
     * 项目流程（富文本）
     */
    @NotBlank(message = "项目流程（富文本）不能为空")
    @Schema(description = "项目流程（富文本）", example = "富文本")
    private String projectFlow;

    /**
     * 申请推广码说明（富文本）
     */
    @NotBlank(message = "申请推广码说明（富文本）不能为空")
    @Schema(description = "申请推广码说明（富文本）", example = "富文本")
    private String applyPromoCodeDesc;

    /**
     * 项目结算周期
     */
    @Min(value = 1, message = "项目结算周期不能小于1")
    @Schema(description = "项目结算周期", example = "2")
    private Integer projectSettlementCycle;

    /**
     * 当前推广人数
     */
    @Min(value = 1, message = "当前推广人数不能小于1")
    @Schema(description = "当前推广人数", example = "150")
    private Integer currentPromotionCount;

    /**
     * 最大推广人数
     */
    @Min(value = 1, message = "当前推广人数不能小于1")
    @Schema(description = "最大推广人数", example = "200")
    private Integer maxPromoterCount;

    /**
     * 项目状态（项目运行|人数已满|项目暂停）
     */
    @EnumValue(enumClass = ProjectStatusEnum.class)
    @Schema(description = "项目状态", example = "项目运行")
    private String projectStatus;

    /**
     * 是否上架
     */
    @Schema(description = "是否上架", example = "true")
    private Boolean isShelves;


    @Serial
    private static final long serialVersionUID = 1L;
}

